use crate::objects::{JumpTable, LetterJumpEntry, PlaylistIndexEntry, ChunkHeader, Database, DataSet, TrackItem, AlbumItem, Playlist};


#[derive(Debug, serde::Serialize)]
pub struct XDatabase {
    pub header: Option<ChunkHeader>,
    pub data: Option<Database>,
    pub children: Vec<XDataSet>
}

#[derive(Debug, serde::Serialize)]
pub struct XDataSet {
    pub header: ChunkHeader,
    pub data: DataSet,
    pub child: XSomeList
}

#[derive(Debug, serde::Serialize)]
pub struct XTrackItem {
    pub header: ChunkHeader,
    pub data: TrackItem,
    pub args: Vec<XArgument>
}

#[derive(Debug, serde::Serialize)]
pub struct XAlbumItem {
    pub header: ChunkHeader,
    pub data: AlbumItem,
    pub args: Vec<XArgument>
}

#[derive(Debug, serde::Serialize)]
pub struct XPlaylist {
    pub header: ChunkHeader,
    pub data: Playlist,
    pub args: Vec<XPlArgument>
}

#[derive(Debug, serde::Serialize)]
pub enum XPlArgument {
    String(XArgument),
    IndexEntry(XPlaylistIndexEntry),
    LetterJumpEntry(XLetterJump),
    RawArgument(Vec<u8>)
}

#[derive(Debug, serde::Serialize)]
pub struct XPlaylistIndexEntry {
    pub data: PlaylistIndexEntry,
    pub v: Vec<u32>
}

#[derive(Debug, serde::Serialize)]
pub struct XLetterJump {
    pub data: LetterJumpEntry,
    pub v: Vec<JumpTable>
}

#[derive(Debug, serde::Serialize)]
pub struct XArgument {
    pub arg_type: u32,
    pub val: String
}

#[derive(Debug, serde::Serialize)]
pub enum XSomeList {
    Playlists(Vec<XPlaylist>),
    TrackList(Vec<XTrackItem>),
    AlbumList(Vec<XAlbumItem>)
}

impl XSomeList {
    fn push_album(&mut self, album: XAlbumItem ) {
        if let XSomeList::AlbumList(albums) = self {
            albums.push(album);
        }
    }

    fn push_track(&mut self, track: XTrackItem ) {
        if let XSomeList::TrackList(tracks) = self {
            tracks.push(track);
        }
    }

    fn push_playlist(&mut self, playlist: XPlaylist ) {
        if let XSomeList::Playlists(playlists) = self {
            playlists.push(playlist);
        }
    }
}

impl XDatabase {
    pub fn find_dataset(&mut self, data_type: u32) -> &mut XDataSet {
        self.children.iter_mut().find(|d| d.data.data_type == data_type).unwrap()
    }
}